<?php

namespace Drupal\standwithukraine\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\standwithukraine\StandWithUkraineSettingsInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure StandWithUkraine message settings for this site.
 */
class StandWithUkraineMessageForm extends StandWithUkraineFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The cache tags invalidator.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  protected $invalidator;

  /**
   * The list of fields that are used in both elements.
   *
   * @var array
   */
  protected $base;

  /**
   * The list of fields that are used in the message element only.
   *
   * @var array
   */
  protected $all;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->invalidator = $container->get('cache_tags.invalidator');
    $instance->renderer = $container->get('renderer');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getName() {
    return 'message';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $this->config = $this->config($this->getEditableConfigNames()[0]);

    $this->alter($form, (array) $this->config->get($this->getName()));
    $this->base = Element::children($form);

    $form['position'] = [
      '#type' => 'radios',
      '#title' => $this->t('Position'),
      '#description' => $this->t('Select on which side of the page a message
should be displayed.'),
      '#options' => [
        StandWithUkraineSettingsInterface::POSITION_LEFT => $this->t('Left'),
        StandWithUkraineSettingsInterface::POSITION_TOP => $this->t('Top (default)'),
        StandWithUkraineSettingsInterface::POSITION_RIGHT => $this->t('Right'),
      ],
      '#default_value' => $this->read('position') ?? '',
    ];

    $form['offset'] = [
      '#type' => 'number',
      '#title' => $this->t('Offset'),
      '#description' => $this->t('How many pixels down the message should be
shifted down the page? When a field is empty then the message will be shifted
down by 20 percent from the page height. Available only for the left or right
position of the message.'),
      '#min' => 0,
      '#field_suffix' => $this->t('pixels'),
      '#default_value' => $this->read('offset'),
      '#states' => [
        'enabled' => [
          ':input[name="position"]' => [
            ['value' => 'left'],
            ['value' => 'right'],
          ],
        ],
      ],
    ];

    $this->all = Element::children($form);

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);
    $values = $form_state->getValues();

    foreach ($this->base as $key) {
      if ($config->get($key) != $values[$key]) {
        $count = $this->entityTypeManager->getStorage('block')
          ->getQuery()
          ->condition('status', TRUE)
          ->condition('plugin', 'standwithukraine_block')
          ->condition('settings.inherit.*', TRUE)
          ->count()
          ->accessCheck(FALSE)
          ->execute();

        if ($count > 0) {
          $this->invalidator->invalidateTags(['standwithukraine']);
        }

        break;
      }
    }

    $lines = &$values['text'];
    $lines = explode(PHP_EOL, $lines);

    array_walk($lines, function (&$line) {
      $line = trim($line);
    });

    if (($count = count($lines)) > 2) {
      $lines = [
        array_slice($lines, 0, $count = round($count / 2)),
        array_slice($lines, $count),
      ];

      array_walk($lines, function (&$line) {
        $line = implode(' ', $line);
      });
    }

    $lines = implode(PHP_EOL, array_filter($lines));

    foreach ($this->all as $key) {
      $config->set($this->getName() . '.' . $key, $values[$key]);
    }

    $config->save();

    parent::submitForm($form, $form_state);
  }

}
