<?php

namespace Drupal\standwithukraine\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\RendererInterface;
use Drupal\standwithukraine\StandWithUkraineTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block to display a message on the background of the flag.
 *
 * @Block(
 *   id = "standwithukraine_block",
 *   admin_label = @Translation("StandWithUkraine"),
 * )
 */
class StandWithUkraineBlock extends BlockBase implements StandWithUkraineBlockInterface {

  use StandWithUkraineTrait;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $config_factory,
    RendererInterface $renderer
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->configFactory = $config_factory;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('renderer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return self::CONFIGURATION + [
      'inherit' => array_fill_keys(
        array_keys(self::CONFIGURATION),
        self::INHERIT
      ),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $link = Link::createFromRoute(
      $this->t('the settings page'),
      'standwithukraine.settings',
      [],
      [
        'attributes' => [
          'target' => '_blank',
        ],
      ]
    );

    $form['inherit'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Inherit'),
      '#description' => $this->t(
        'Re-use value from the field on the @url where this field has the same
title as the checkbox of the current field.',
        ['@url' => $link->toString()]
      ),
      '#default_value' => array_keys(
        array_filter($this->configuration['inherit'])
      ),
    ];

    $this->alter($form, $this->configuration);

    foreach (array_keys(self::CONFIGURATION) as $name) {
      $form[$name]['#states'] = [
        'disabled' => [
          ':input[name="settings[inherit][' . $name . ']"]' => [
            'checked' => TRUE,
          ],
        ],
      ];

      $form['inherit']['#options'][$name] = $form[$name]['#title'];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    foreach (array_merge(['inherit'], array_keys(self::CONFIGURATION)) as $key) {
      $this->configuration[$key] = $form_state->getValue($key);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->configFactory->get('standwithukraine.settings');
    $options = [];

    foreach (self::CONFIGURATION as $key => $default_value) {
      $global_value = $config->get('message.' . $key);

      $options['#' . $key] = $this->configuration['inherit'][$key] &&
        ($default_value !== NULL || !empty($global_value))
        ? $global_value : $this->configuration[$key];
    }

    return [
      '#theme' => 'standwithukraine',
      '#attached' => [
        'library' => ['standwithukraine/common'],
      ],
    ] + $options;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return Cache::mergeTags(parent::getCacheTags(), ['standwithukraine']);
  }

}
