<?php

namespace Drupal\standwithukraine\Service;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StreamWrapper\PublicStream;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Theme\ThemeManagerInterface;

/**
 * Defines the favicon service.
 */
class StandWithUkraineFavicon extends StandWithUkraineImage implements StandWithUkraineFaviconInterface {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    CacheBackendInterface $cache,
    LoggerChannelFactoryInterface $logger_factory,
    PublicStream $stream,
    TranslationInterface $translation,
    ThemeManagerInterface $theme,
    FileUrlGeneratorInterface $file_url_generator = NULL,
    ConfigFactoryInterface $config_factory = NULL
  ) {
    parent::__construct(
      $cache,
      $logger_factory,
      $stream,
      $translation,
      $theme,
      $file_url_generator
    );

    $this->configFactory = $config_factory ?: \Drupal::configFactory();
  }

  /**
   * {@inheritdoc}
   */
  public function build($theme) {
    if (
      ($items = $this->cached($root = 'favicon', $theme)) !== NULL ||
      ($filename = theme_get_setting($root . '.url', $theme)) === NULL
    ) {
      return (array) $items;
    }

    $config = $this->configFactory->get('standwithukraine.settings');
    $settings = (array) $config->get($root . '.themes.' . $theme);

    return $this->get(
      $theme,
      $filename,
      $root,
      $settings['ratio'] ?? self::UNDEFINED_RATIO,
      !empty($settings['background']),
      FALSE
    );
  }

  /**
   * {@inheritdoc}
   */
  public function isSupported($type) {
    foreach (self::TYPES as $types) {
      if (in_array($type, $types)) {
        return TRUE;
      }
    }

    return FALSE;
  }

}
