<?php

namespace Drupal\standwithukraine\Service;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StreamWrapper\PublicStream;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Theme\ThemeManagerInterface;

/**
 * Defines the favicon service interface.
 */
interface StandWithUkraineFaviconInterface extends StandWithUkraineImageInterface {

  /**
   * The default ratio for a known theme.
   *
   * Use a specific ratio for themes that are provided by Drupal core or someone
   * else who is using favicon in Drupal logo style (image of drop).
   */
  const DEFINED_RATIO = 60;

  /**
   * The predefined themes list.
   */
  const THEMES = [
    'bartik',
    'bootstrap',
    'claro',
    'gin',
    'olivero',
    'seven',
    'stark',
  ];

  /**
   * StandWithUkraineFavicon constructor.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger channel factory.
   * @param \Drupal\Core\StreamWrapper\PublicStream $stream
   *   The public stream wrapper.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   The string translation.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $theme
   *   The theme manager.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface|null $file_url_generator
   *   (optional) The file URL generator. Defaults to NULL.
   * @param \Drupal\Core\Config\ConfigFactoryInterface|null $config_factory
   *   (optional) The configuration factory. Defaults to NULL.
   */
  public function __construct(
    CacheBackendInterface $cache,
    LoggerChannelFactoryInterface $logger_factory,
    PublicStream $stream,
    TranslationInterface $translation,
    ThemeManagerInterface $theme,
    FileUrlGeneratorInterface $file_url_generator = NULL,
    ConfigFactoryInterface $config_factory = NULL
  );

  /**
   * Extracts images from favicon and style them in Ukraine flag colors.
   *
   * @param string $theme
   *   The theme name.
   *
   * @return array
   *   An array of arrays. Each key is the theme name (e.g. seven), and each
   *   value is an array of paths to images, keyed by dimensions (e.g. 32x32).
   */
  public function build($theme);

}
